@@ -3,17 +3,27 @@ import re
 import traceback
 from collections.abc import Collection
 from functools import wraps
-from types import MethodType
+from types import MethodType, TracebackType
+from typing import Any, Callable
+from typing import Collection as TypingCollection
+from typing import Optional, Pattern, Tuple, Type, TypeVar, Union, cast
 
 from django.core.cache import DEFAULT_CACHE_ALIAS, caches
 
-from django_perf_rec.operation import AllSourceRecorder, Operation
+from django_perf_rec.operation import AllSourceRecorder, BaseRecorder, Operation
 
 
 class CacheOp(Operation):
-    def __init__(self, alias, operation, key_or_keys, traceback):
+    def __init__(
+        self,
+        alias: str,
+        operation: str,
+        key_or_keys: Union[str, TypingCollection[str]],
+        traceback: traceback.StackSummary,
+    ):
         self.alias = alias
         self.operation = operation
+        cleaned_key_or_keys: Union[str, TypingCollection[str]]
         if isinstance(key_or_keys, str):
             cleaned_key_or_keys = self.clean_key(key_or_keys)
         elif isinstance(key_or_keys, Collection):
@@ -24,7 +34,7 @@ class CacheOp(Operation):
         super().__init__(alias, cleaned_key_or_keys, traceback)
 
     @classmethod
-    def clean_key(cls, key):
+    def clean_key(cls, key: str) -> str:
         """
         Replace things that look like variables with a '#' so tests aren't
         affected by random variables
@@ -33,7 +43,7 @@ class CacheOp(Operation):
             key = var_re.sub("#", key)
         return key
 
-    VARIABLE_RES = (
+    VARIABLE_RES: Tuple[Pattern[str], ...] = (
         # Django session keys for 'cache' backend
         re.compile(r"(?<=django\.contrib\.sessions\.cache)[0-9a-z]{32}\b"),
         # Django session keys for 'cached_db' backend
@@ -46,11 +56,11 @@ class CacheOp(Operation):
         re.compile(r"\d+"),
     )
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         return super().__eq__(other) and self.operation == other.operation
 
     @property
-    def name(self):
+    def name(self) -> str:
         name_parts = ["cache"]
         if self.alias != DEFAULT_CACHE_ALIAS:
             name_parts.append(self.alias)
@@ -58,31 +68,33 @@ class CacheOp(Operation):
         return "|".join(name_parts)
 
 
-class CacheRecorder:
+CacheFunc = TypeVar("CacheFunc", bound=Callable[..., Any])
+
+
+class CacheRecorder(BaseRecorder):
     """
     Monkey patches a cache class to call 'callback' on every operation it calls
     """
 
-    def __init__(self, alias, callback):
-        self.alias = alias
-        self.callback = callback
-
-    def __enter__(self):
+    def __enter__(self) -> None:
         cache = caches[self.alias]
 
-        def call_callback(func):
+        def call_callback(func: CacheFunc) -> CacheFunc:
             alias = self.alias
             callback = self.callback
 
             @wraps(func)
-            def inner(self, *args, **kwargs):
+            def inner(self: Any, *args: Any, **kwargs: Any) -> Any:
                 # Ignore operations from the cache class calling itself
 
                 # Get the self of the parent via stack inspection
                 frame = inspect.currentframe()
+                assert frame is not None
                 try:
                     frame = frame.f_back
-                    is_internal_call = frame.f_locals.get("self", None) is self
+                    is_internal_call = (
+                        frame is not None and frame.f_locals.get("self", None) is self
+                    )
                 finally:
                     # Always delete frame references to help garbage collector
                     del frame
@@ -100,14 +112,19 @@ class CacheRecorder:
 
                 return func(*args, **kwargs)
 
-            return inner
+            return cast(CacheFunc, inner)
 
         self.orig_methods = {name: getattr(cache, name) for name in self.cache_methods}
         for name in self.cache_methods:
             orig_method = self.orig_methods[name]
             setattr(cache, name, MethodType(call_callback(orig_method), cache))
 
-    def __exit__(self, _, __, ___):
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_value: Optional[BaseException],
+        exc_traceback: Optional[TracebackType],
+    ) -> None:
         cache = caches[self.alias]
         for name in self.cache_methods:
             setattr(cache, name, self.orig_methods[name])
