@@ -1,6 +1,6 @@
 in_pytest = False
 
 
-def pytest_configure(config):
+def pytest_configure() -> None:
     global in_pytest
     in_pytest = True
